#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <Inventor/SoDB.h>
#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoRotationXYZ.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/sensors/SoTimerSensor.h>

int create_socket(int *, int, int);
void TimerCallback(void *, SoSensor *);

SoTransform *cube_transform;
SoTransform *cylinder_transform;
SoMaterial *material;
SoMaterial *reflection;

int descriptor;

float red[91];
float green[91];
float blue[91];

int main(int argc, char **argv) {
	int i, port;
	FILE *fp;

	Widget window;
	SoSeparator *root;
	SoInput *SceneInput;
	SoTimerSensor *TimerSensor;
	SoXtExaminerViewer *viewer;

/*
 *      Parse command line arguments and create a socket
 *
 */

	for (i=1;i<argc - 1;i++) {
		if (strcmp(*(argv+i),"-p") == 0) {      // Port switch
			i++;
			port = atoi(*(argv + i));       // Set port to the command line argument
		}
	}

	if (!port) {
		printf("Usage: %s -p port\n\n",*argv);
		printf("\t-p port\t\tport to listen for incomming clients\n");
		exit(0);
	}

	if (create_socket(&descriptor, port, 1) <= 0) {
		printf("Error: Unable to create socket\n");
		exit(0);
	}

/*
 *      Arguments Parsed, Socket Created, Code Continues...
 *
 */

	/* Read RGB color values (of the rainbow) from file */
	fp = fopen("COLOR_PATTERNS", "r");
	for (i=0;i<91;i++) fscanf(fp,"%f%f%f", &red[i], &green[i], &blue[i]);
	fclose(fp);

	/* Initialize Inventor and Xt; create new viewer */
	window = SoXt::init(*argv);
	viewer = new SoXtExaminerViewer(window);

	/* Parse the Inventor file */
	SceneInput = new SoInput;
	SceneInput->openFile("gview.iv");
	root = new SoSeparator;
	root = SoDB::readAll(SceneInput);
	SceneInput->closeFile();

	/* Create pointers to named nodes */
	cube_transform = (SoTransform *) SoNode::getByName("cube_translation");
	cylinder_transform = (SoTransform *) SoNode::getByName("cylinder_translation");
	material = (SoMaterial *) SoNode::getByName("material");
	reflection = (SoMaterial *) SoNode::getByName("reflection");

	/* Display SceneGraph in viewer */
	viewer->setSceneGraph(root);
	viewer->show();
	
	/* Schedule invocation of a function with void* arguments */
	TimerSensor = new SoTimerSensor(TimerCallback, NULL);
	TimerSensor->schedule();

	/* Display viewer window and initialize event handler */
	SoXt::show(window);
	SoXt::mainLoop();
}

void TimerCallback(void *arguments, SoSensor *sensor) {
	int connection, color, glow;
	float cylinder_translation, cube_translation, scale, shininess;

	/* Read values from LabView */
	read(descriptor, (char*) &cube_translation, sizeof(float));
	read(descriptor, (char*) &cylinder_translation, sizeof(float));
	read(descriptor, (char*) &scale, sizeof(float));
	read(descriptor, (char*) &shininess, sizeof(float));
	read(descriptor, (char*) &color, sizeof(int));
	read(descriptor, (char*) &glow, sizeof(int));

	/* Modify scene with values from Labview */
	cube_transform->translation.setValue(cube_translation, 0, 0);	
	cylinder_transform->translation.setValue(cylinder_translation, 0, 0);	

	cube_transform->scaleFactor.setValue(scale,scale,scale);	

	material->diffuseColor.setValue(red[color], green[color], blue[color]);
	material->emissiveColor.setValue(red[glow], green[glow], blue[glow]);

	reflection->shininess.setValue(shininess);

	/* Check if Labview has disconnected */
	read(descriptor, (char*) &connection, sizeof(int));
	write(descriptor, (char*) &connection, sizeof(int));

	if (!connection) {
		close(descriptor);
		exit(0);
	}

	/* Schedule another invocation of the function */
	sensor->schedule();
}
